/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("rmps.core.projects.ProjectClient");

dojo.require("rmps.core.RootServices");

dojo.require("jazz.client.xhr");

var PROJECT_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "projects");

/**
 * Client class for working the DM Projects front service.
 */
rmps.core.projects.ProjectClient = {

	/**
	 * Returns a deferred object to fetch the property of the project.  Both parameters
	 * are required.  
	 */
	getProjectProperty: function(projectId, propertyId) {
		var query = {};
		query.projectId = projectId;
		var args = {
			url: PROJECT_PATH + "/properties/" + projectId + "/" + propertyId,
			headers: {
				"Accept": "text/plain",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "text",
			load: function(response, ioArgs) {
				
			}
		};
		return jazz.client.xhrGet(args);	
	},
	
	/**
	 * Makes a request to set the value of a property for this project.
	 * projectId (required)
	 * propertyId (required)
	 * propertyValue - the property value as a string, or null to indicate that the property should be unset
	 * isReference (optional) - true if the property represents a URL reference; false otherwise.  Defaults to false
	 */
	setProjectProperty: function(projectId, propertyId, propertyValue, isReference) {
		var query = {};
		query.value = propertyValue ? propertyValue : ""; //use empty string to unset the property
		
		if (isReference) {
			query.isReference = isReference;
		}
		
		var args = {
			url: PROJECT_PATH + "/properties/" + projectId + "/" + propertyId + "?" + dojo.objectToQuery(query),
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			load: function(response, ioArgs) {
				return response;
			}		
		};
		return jazz.client.xhrPost(args);	
	}

} 
})();