/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.ListWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("jazz.ui.ListBox");
dojo.require("jazz.ui.FilterBox");
dojo.require("rmps.ui.queryeditor.IQueryWidget");
dojo.require("rmps.ui.queryeditor.QueryConstants");

dojo.requireLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

(function(){

var messages = dojo.i18n.getLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");
var QueryConstants = rmps.ui.queryeditor.QueryConstants;

dojo.declare("rmps.ui.queryeditor.ListWidget", [dijit._Widget, dijit._Templated, rmps.ui.queryeditor.IQueryWidget], {
	templatePath: dojo.moduleUrl("rmps.ui", "queryeditor/templates/ListWidget.html"),

	_queryConditionListProperty: null,
	_queryConditionName: null,
	_init: true,
	_defWidgetData: null,
	_filterBoxWidget: null,
	_listBoxWidget: null,
	_multiple: false,
	_selection: null,
	
	onClose: function(e) {
		this.inherited(arguments);
		this.destroy();
		return true;
	},
	
	postCreate: function() {
		this._filterBoxWidget = new jazz.ui.FilterBox({
			onChange: dojo.hitch(this, this._onFilterChange)
		});
		this._filterBox.appendChild(this._filterBoxWidget.domNode);
		this._listBoxWidget = new jazz.ui.ListBox({
			onClick: dojo.hitch(this, this._onSelectionChange),
			allowMultipleSelections: this._multiple,
			margin: "0px 2px 2px 2px",
			height: "100px",
			sortOrder: 2
		});
		this._listBox.appendChild(this._listBoxWidget.domNode);
		
		this.refreshWidgetData();
		
		this._warningMessage.style.display = 'none';
		
		this.inherited(arguments);
	},
	
	destroy: function() {
		if (this._filterBoxWidget) {
			this._filterBoxWidget.destroy();
			delete this._filterBoxWidget;
		}
		if (this._listBoxWidget) {
			this._listBoxWidget.destroy();
			delete this._listBoxWidget;
		}
		
		this.inherited(arguments);
	},
		
	refreshWidgetData: function() {
		
		this._listBoxWidget.showLoading(true);
		
		this._selection = [];
		var conditionData = this.getConditionData();
		if (conditionData) {
			this._selection = conditionData[this._queryConditionListProperty];
		}
		
		this._clearValues();
		
		// Assuming compatible data format		
		var store = this.getDataStore();
		var def = this._defWidgetData = store.getWidgetData(this.getIdentifier(), this.getWidgetDataOptions());
		def.addCallback(this, function (data) {
			this._onWidgetDataReady(data);
		});
		def.addBoth(this, function() {
			this._listBoxWidget.showLoading(false);
		});
	},
	
	_onWidgetDataReady: function (data, selection) {
		this._addValues(data);
		this._setSelection(this._selection);
		this._init = false;
	},

	setEnabled: function(value) {
		dojo.style(this._listBox, "disabled", !value);
		this._filterBoxWidget.setEnabled(value);
	},
	
	setMultiple: function(multiple) {
		this._multiple = multiple;
		if (this._listBoxWidget) {
			this._listBoxWidget.allowMultipleSelections = multiple;	
		}
	},
		
	getSelectedValues: function() {
		var selection = this._listBoxWidget.getSelection();
		return dojo.map(selection, function(item) {
			return {
				id: item.id,
				targetType: item.value
			};
		});
	},
	
	addOnSelectionChangeHandler: function(context, method) {
		if (context && method) {
			dojo.connect(this._listBoxWidget, "onClick", context, method);
		}
	},
	
	_onFilterChange: function(value) {
		var filterList = this._listBoxWidget;
		var elements = filterList.getAllEntries();
		var selection = {};
		dojo.forEach(filterList.getSelection(), function(entry) {
			selection[entry.id] = 1;
		});
		filterList.setSelection(filterList.getSelection(), false);
		var newSelection = [];
		var firstVisible;
		for (var i = 0, len = elements.length; i < len; i++) {
			if (this._shouldFilterElement(elements[i], value)) {
				filterList.showEntry(elements[i], false);
			} else {
				if (elements[i].id in selection) {
					newSelection.push(elements[i]);
				}
				if (!firstVisible) { firstVisible = elements[i]; }
				filterList.showEntry(elements[i], true);
			}
		}
	},
		
	_shouldFilterElement: function(elt, value) {
		return elt.label.toLowerCase().indexOf(value) === -1;
	},

	_onSelectionChange: function(entry) {
		// TODO: Support removeDirty as well
		if (!this._init) {
			this.mainWidget.onMarkDirty("listSelection");
		}
	},
	
	_clearValues: function() {
		this._listBoxWidget.removeAllEntries();
	},
	
	_addValues: function(values) {
		if (!values || values.length === 0) {
			return;
		}
		
		var entries = dojo.map(values, function(value) {
			return {
				label: value.label,
				id: value.id,
				value: value.targetType,
				type: 0
			};
		});
		
		this._listBoxWidget.addEntries(entries);
	},
	
	_setSelection: function(selection) {
		var w = this._listBoxWidget; 
		w.setSelection(w.getAllEntries(), false);
		
		if (!selection || selection.length === 0) {
			return;
		}
		
		var entries = dojo.map(selection, function(selection) {
			return {
				id: selection
			};
		});
		
		w.setSelection(entries, true);
	},
		
	preSerialize: function() {
		return this._defWidgetData;
	},
	
	serialize: function() {
		var obj = {};
		var condition = obj[this._queryConditionName] = {};
		condition[this._queryConditionListProperty] = dojo.map(this.getSelectedValues(), function(item) {
			return item.id;
		});
		condition[QueryConstants.CONDITION_PRESENTATION_PROPERTY] = this.getIdentifier();
		condition[QueryConstants.CONDITION_IGNORE_CONTEXT_PROPERTY] = false;
		return obj;
	},
	
	validate: function (rootTab) {
		var valResult = { 
			hasNonContainerPortlet: true,
			errors: []
		};
		var obj = this.serialize();
		var list = obj[this._queryConditionName];
		if (list[this._queryConditionListProperty].length === 0) {
			var str = dojo.string.substitute(messages.valEmptyDataNotAllowed, [this._queryConditionName, this._queryConditionListProperty]);
			valResult.errors.push(str);
		}
		return valResult;
	},
	
	_showWarning: function (message) {
		while (this._warningMessage.hasChildNodes()) {
			this._warningMessage.removeChild(this._warningMessage.childNodes[0]);
		}
		
		if (message) {
			this._warningMessage.appendChild(document.createTextNode(message));
			this._warningMessage.style.display = '';
		} else {
			this._warningMessage.style.display = 'none';
		}
	}
});
}());