/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.filters.UserFilterWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("jazz.ui.ListBox");
dojo.require("jazz.ui.FilterBox");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog");
dojo.require("rmps.core.user.UserClient");
dojo.require("rmps.ui.queryeditor.filters.IResourcesFilter");
dojo.require("rmps.ui.queryeditor.QueryConstants");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

(function(){

var ServiceResponseHandler= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var SelectContributorDialog = com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog;
var UserClient = rmps.core.user.UserClient;
var QueryConstants = rmps.ui.queryeditor.QueryConstants;
var Messages = dojo.i18n.getLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

dojo.declare("rmps.ui.queryeditor.filters.UserFilterWidget", [dijit._Widget, dijit._Templated, rmps.ui.queryeditor.filters.IResourcesFilter], {

	templatePath: dojo.moduleUrl("rmps.ui", "queryeditor/filters/templates/UserFilterWidget.html"),
	
	_hasSelection: false,
	_usersPromise: null,
	_selection: null,
	_listBoxWidget: null,
	_filterBoxWidget: null,

	// since initialization is performed in async manner, we need to know how much is left to be initialized
	_initSema: 0,
	
	constructor: function() {
		this._selection = [];
	},
	
	postCreate: function() {
		this._filterBoxWidget = new jazz.ui.FilterBox({
			onChange: dojo.hitch(this, this._onFilterChange)
		});
		this._filterBox.appendChild(this._filterBoxWidget.domNode);
		this._listBoxWidget = new jazz.ui.ListBox({
			onClick: dojo.hitch(this, this._onSelectionChange),
			margin: "2px",
			height: "100px"
		});
		this._listBox.appendChild(this._listBoxWidget.domNode);
		
		this._createAddContributorButton();
		
		this._initSema++; // _initValues
		this._usersPromise = UserClient.getUsers(dojo.getObject("jazz.app.currentApplication").context).then(dojo.hitch(this, this._initValues));
	},
	
	destroy: function() {
		try {
			if(this.childButton) {
				this.childButton.destroy();
			}
			this._destroyDialog();
			
			if (this._filterBoxWidget) {
				this._filterBoxWidget.destroy();
				delete this._filterBoxWidget;
			}
			if (this._listBoxWidget) {
				this._listBoxWidget.destroy();
				delete this._listBoxWidget;
			}
			
			this.inherited(arguments);
		} catch (e) {
			
		}
	},
	
	/* String[] */ getResources: function() {
		
		if (!this._hasSelection) {
			return this._getInitialSelectionValues();			
		}
			
		return dojo.map(this._listBoxWidget.getSelection(), function(entry) {
			return entry.id;
		});
	},
	
	setSelection: function(selection) {
		this._initSema++;
		this._usersPromise.then(dojo.hitch(this, function() {
			this._selection = selection;
			this._setSelection();
			this._initSema--;
		}));
	},
	
	setResources: function(value) {
		this.setSelection(value);
	},
	
	getDataType: function() {
		return QueryConstants.USER_DATATYPE;
	},
	
	validate: function(valResult) {
		if (this.getResources().length === 0) {
			valResult.errors.push(Messages.valNoUserSelected);
		}
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Internals

	
	_getInitialSelectionValues: function() {
		var values = this._selection || [];
		var selection = [];
		for (var i = 0; i < values.length; i++) {
			selection.push(values[i]);
		}
		return selection;
	},
		
	_onSelectionChange: function() {
		this._hasSelection = true;
		if (!this._initSema && this.changeCallback) {
			this.changeCallback();
		}
	},
	
	_createAddContributorButton: function() {
		var self = this;
		var addUserButtonParameters = {
			label: Messages.labelAddUser,
			onClick: function() {
				self._handleAddUser();
			}
		};
		var addUserButton = new dijit.form.Button(addUserButtonParameters);
		this._addUserButton.appendChild(addUserButton.domNode);
		
		this.childButton = addUserButton; 
	},
	
	_handleAddUser: function() {
		this._destroyDialog();
		var self= this;
		var params = {
				title: Messages.dialogTitleAddUsers,
				parentElement: this.domNode,
				disableUsers: [],
				onOK: function(selectedContribs) {
					self._addUsers(dojo.map(selectedContribs, function(contrib) {
						return {
							label: contrib.name,
							id: UserClient.getUserUri(contrib.userId),
							value: null,
							type: 0
						};
					}), true);
				},
				onClose: function(selectedContribs) {
					self._destroyDialog();
				}	
			};
		this.userListWidget = new SelectContributorDialog(params);
			
	},
	_destroyDialog: function() {
		if (this.userListWidget) {
			this.userListWidget.destroy(false);
		}
		this.userListWidget = null;
	},		
	_addUsers: function(selection, select) {
		var entries = [];
		var newentries = [];
		var defs = [];
		var addUser = function(user) {
			var entry = {
				label: user.name,
				id: user.uri,
				value: null,
				type: 0
			};
			entries.push(entry);
			newentries.push(entry);
		};
		var w = this._listBoxWidget;
		for (var i= 0; i < selection.length; i++) {
			var selected = selection[i];
			var entry = w.getEntry(selected.uri);
			if (entry) {
				entries.push(entry);
			} else {
				if (selected instanceof rmps.core.user.User) {
					addUser(selected);
				} else {
					var def = UserClient.getUser(selected.id || selected.uri || selected);
					def.then(addUser);
					defs.push(def);					
				}
			}
		}
		if (defs.length > 0) {
			var defList = new dojo.DeferredList(defs);
			defList.then(function() {
				w.addEntries(newentries);
				if (select) {
					w.setSelection(entries, true);
				}
			});
		} else {
			w.addEntries(newentries);
			if (select) {
				w.setSelection(entries, true);	
			}
		}
		
		if (!this._initSema && this.changeCallback) {
			this.changeCallback();
		}
	},
	
	_initValues: function(users) {
		if (!users || users.length === 0) {
			return false;
		}
		this._listBoxWidget.removeAllEntries();
		this._addUsers(users, false);
		this._initSema--;
		return true;
	},
	
	_setSelection: function() {
		if (!this._selection) {
			return;
		}
		if (this._selection.length > 0) {
			this._hasSelection = true;			
		}
		// Unselect all
		this._listBoxWidget.setSelection(this._listBoxWidget.getAllEntries(), false);
		this._usersPromise.then(dojo.hitch(this, function() {
			this._addUsers(this._selection, true);
		}));
	},
	
	_onFilterChange: function(value) {
		var filterList = this._listBoxWidget;
		var elements = filterList.getAllEntries();
		var selection = {};
		dojo.forEach(filterList.getSelection(), function(entry) {
			selection[entry.id] = 1;
		});
		filterList.setSelection(filterList.getSelection(), false);
		var newSelection = [];
		var firstVisible;
		for (var i = 0, len = elements.length; i < len; i++) {
			if (this._shouldFilterElement(elements[i], value)) {
				filterList.showEntry(elements[i], false);
			} else {
				if (elements[i].id in selection) {
					newSelection.push(elements[i]);
				}
				if (!firstVisible) { firstVisible = elements[i]; }
				filterList.showEntry(elements[i], true);
			}
		}
	},
		
	_shouldFilterElement: function(elt, value) {
		return elt.label.toLowerCase().indexOf(value) === -1;
	}
});
	
}());

